pub struct PlaceKey {
pub base: PlaceBaseKey,
pub fields: Vec<usize>,
}Expand description
Projection-insensitive enough place key for relevance tracking.
Fields
base: PlaceBaseKeyBase local/argument of the place.
fields: Vec<usize>Field projections kept from the contract place.
Implementations
Sourceimpl PlaceKey
impl PlaceKey
Sourcepub fn from_mir_place(place: &Place<'_>) -> Self
pub fn from_mir_place(place: &Place<'_>) -> Self
Build a relevance place key from a MIR place.
Sourcepub fn local(&self) -> Option<Local>
pub fn local(&self) -> Option<Local>
Return the MIR local represented by this key when it is already known.
Sourcepub fn from_origin(local: usize, fields: Vec<usize>) -> Self
pub fn from_origin(local: usize, fields: Vec<usize>) -> Self
Build a PlaceKey from an analysis-level (origin_local, fields) tuple.
Sourceimpl PlaceKey
impl PlaceKey
Sourcepub fn from_contract_place(place: &ContractPlace<'_>) -> Self
pub fn from_contract_place(place: &ContractPlace<'_>) -> Self
Build a relevance place key from a parsed contract place.